/* Qaquarelle
 * Copyright (C) 2009 Valeriy Fedotov <valeriy.fedotov@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "toolpanel.h"

#include <QGridLayout>
#include "iSkin.h"
#include "commdefs.h"

ToolPanel::ToolPanel( QWidget * parent, Qt::WindowFlags f )
    : QWidget ( parent , f)
{
    m_Id = ISpawnedObject::allocateId ();

    QDir mypath (DATA_PATH);
    QDir iconDir (mypath.absoluteFilePath("images/gui/icons"));
    m_group = new QButtonGroup(this);
    m_penToolButton = new QToolButton(this);
    m_penToolButton->setIcon(QIcon( iconDir.absoluteFilePath("icon_brush.png")));
    m_penToolButton->setCheckable(true);
    m_group->addButton(m_penToolButton);
    m_dropperToolButton = new QToolButton(this);
    m_dropperToolButton->setIcon(QIcon( iconDir.absoluteFilePath("icon_dropper.png")));
    m_dropperToolButton->setCheckable(true);
    m_group->addButton(m_dropperToolButton);
    m_selectionToolButton = new QToolButton(this);
    m_selectionToolButton->setIcon(QIcon( iconDir.absoluteFilePath("icon_selection.png")));
    m_selectionToolButton->setCheckable(true);
    m_group->addButton(m_selectionToolButton);
    m_group->setId (m_penToolButton, 0);
    m_group->setId (m_dropperToolButton, 1);
    m_group->setId (m_selectionToolButton, 2);
    m_group->setExclusive (true);

    QObject::connect (m_group, SIGNAL(buttonReleased(int)), this, SLOT(buttonChange(int)));
    m_penToolButton->setChecked (true);

    m_gridLayout = new QGridLayout(this);
    m_gridLayout->addWidget( m_penToolButton );
    m_gridLayout->addWidget( m_dropperToolButton );
    m_gridLayout->addWidget( m_selectionToolButton );
    m_gridLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(m_gridLayout);

    m_Connector   = NULL;
    m_iEyedropper = NULL;
    m_iSelection  = NULL;
}

ToolPanel::~ToolPanel()
{
    //delete m_penToolButton;
    //delete m_dropperToolButton;
    //delete m_gridLayout;

    SAFE_RELEASE (m_iEyedropper);
    SAFE_RELEASE (m_iSelection);
    SAFE_RELEASE (m_Connector);
}

void ToolPanel::connect (IConnector * caller) {
    SAFE_RELEASE (m_Connector);

    if (caller) {
        m_Connector = caller;
        m_Connector->addRef ();

        // Request the instruments list.
        m_Connector->allocateObject (this, 14, ENTITY_INSTRUMENT_PLUGIN);
    }
}

void ToolPanel::complitedRequest (int param, int length, void * data) {
    Q_UNUSED (length);

    if (param == 14) {
        IInstrumentProxy * instrument = static_cast<IInstrumentProxy*>(data);
        if (instrument) {
            // Accept only tool instruments,
            if (instrument->instrumentGroup () == INSTRUMENTGROUP_TOOLBOX) {
                if (0 == strcmp (instrument->myClassName (), "Server::Eyedropper")) {
                    SAFE_RELEASE (m_iEyedropper);
                    m_iEyedropper = g_fac_SPAWN_INSTRUMENT_PROXY (instrument);
                } else
                if (0 == strcmp (instrument->myClassName (), "Server::Selection")) {
                    SAFE_RELEASE (m_iSelection);
                    m_iSelection = g_fac_SPAWN_INSTRUMENT_PROXY (instrument);
                }
            }

            instrument->release ();
        }
    }
}

void ToolPanel::buttonChange (int id) {
    if (id == 0)
        emit instrumentChanged (NULL);
    else
    if (id == 1 && m_iEyedropper != NULL)
        emit instrumentChanged (m_iEyedropper);
    else
    if (id == 2 && m_iSelection != NULL)
        emit instrumentChanged (m_iSelection);
}
