/* Qaquarelle
 * Copyright (C) 2009 Valeriy Fedotov <valeriy.fedotov@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef TOOLPANEL_H
#define TOOLPANEL_H

#include "commdefs.h"
#include "commtypes.h"

#include <QWidget>
#include <QToolButton>
#include "iConnector.h"
#include "iInstrumentProxy.h"


class ToolPanel : public QWidget,
                  public IConnectorCallback
{
    Q_OBJECT

signals:
    /**
     * Emitted when user selects
     * a different instrument.
     */
    void instrumentChanged (IInstrumentProxy * instrument);

public:
    ToolPanel( QWidget * parent = 0, Qt::WindowFlags f = 0 );
    ~ToolPanel();

    /***** ICONNECTORCALLBACK IMPLANTS *****/

    /**
     * Pass the connector interface
     * to the object.
     */
    virtual void connect (IConnector * caller);

    /**
     * A callback, called by connector when
     * the requested data is recieved.
     */
    virtual void complitedRequest (int param, int length, void * data);

    /****** ISPAWNEDOBJECT IMPLANTS *******/

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    virtual void addRef (void) {
    }

    virtual void release (void) {
    }

    /**
     * Ask the object to emit all
     * notification signals if any.
     */
    virtual void pokeObject (void) {
    }

    virtual const char * myClassName (void) {
        return "Client::ToolPanel";
    }

private slots:
    void buttonChange (int id);

private:
    QToolButton  *m_penToolButton;
    QToolButton  *m_dropperToolButton;
    QToolButton  *m_selectionToolButton;
    QButtonGroup *m_group;
    QLayout *m_gridLayout;

    int          m_Id;
    IConnector * m_Connector;

    // Tool instruments.

    IInstrumentProxy * m_iEyedropper;   /**< Eyedropper instrument. */
    IInstrumentProxy * m_iSelection;    /**< Selection instrument.  */
};

#endif // TOOLPANEL_H
