/* Qaquarelle
 * Copyright (C) 2009 Vasiliy Makarov <drmoriarty.0@gmail.com>
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "preferencesdialog.h"
#include "preferences.h"

PreferencesDialog::PreferencesDialog(QWidget* parent) : QDialog(parent)
{
	ui.setupUi(this);
	ui.checkBox_smartStroke->setChecked(PREF.smartStroke);
	ui.comboBox_colorPickerStyle->setCurrentIndex(PREF.colorPickerStyle);
	ui.key_wheelScaleBrush->setKey(PREF.wheelKeys[WheelScalesBrush]);
	ui.key_wheelScaleCanvas->setKey(PREF.wheelKeys[WheelScalesCanvas]);
	ui.key_dragMoveCanvas->setKey(PREF.dragKeys[DRAGSTATE_CANVAS_MOVE]);
	ui.key_dragScaleCanvas->setKey(PREF.dragKeys[DRAGSTATE_CANVAS_SCALE]);
	ui.key_dragRotateCanvas->setKey(PREF.dragKeys[DRAGSTATE_CANVAS_ROTATE]);
	ui.key_dragScaleBrush->setKey(PREF.dragKeys[DRAGSTATE_BRUSH_SCALE]);
	ui.key_dragRotateBrush->setKey(PREF.dragKeys[DRAGSTATE_BRUSH_ROTATE]);
	ui.groupBox_pressure->setChecked(PREF.pressureControl);
	minPr = PREF.pressureMin;
	maxPr = PREF.pressureMax;

    if (PREF.pressureControl) {
        ui.pressureBox_1->setPressure (minPr);
        ui.pressureBox_2->setPressure (maxPr);
    }

    ui.textureCompression->setValue (PREF.ogl_textureCompression);
	ui.strokeQuality->setValue(PREF.strokeQuality);
}

void PreferencesDialog::storeChanges()
{
	PREF.smartStroke = ui.checkBox_smartStroke->isChecked();
	PREF.colorPickerStyle = ui.comboBox_colorPickerStyle->currentIndex();
	PREF.wheelKeys[WheelScalesBrush] = ui.key_wheelScaleBrush->key();
	PREF.wheelKeys[WheelScalesCanvas] = ui.key_wheelScaleCanvas->key();
	PREF.dragKeys[DRAGSTATE_CANVAS_MOVE] = ui.key_dragMoveCanvas->key();
	PREF.dragKeys[DRAGSTATE_CANVAS_SCALE] = ui.key_dragScaleCanvas->key();
	PREF.dragKeys[DRAGSTATE_CANVAS_ROTATE] = ui.key_dragRotateCanvas->key();
	PREF.dragKeys[DRAGSTATE_BRUSH_SCALE] = ui.key_dragScaleBrush->key();
	PREF.dragKeys[DRAGSTATE_BRUSH_ROTATE] = ui.key_dragRotateBrush->key();
	PREF.pressureControl = ui.groupBox_pressure->isChecked();

    // [castedflake] I simplified the interface a little.
    acceptPressure ();

    PREF.ogl_textureCompression = ui.textureCompression->value ();
	PREF.strokeQuality = ui.strokeQuality->value();
}

void PreferencesDialog::acceptPressure()
{
    if (ui.groupBox_pressure->isChecked()) {
        minPr = ui.pressureBox_1->pressure();
        maxPr = ui.pressureBox_2->pressure();

        // [castedflake] Save if the range is valid.
        if (maxPr - minPr > 0.1f) {
            PREF.pressureMin = minPr;
            PREF.pressureMax = maxPr;
        } else {
            PREF.pressureControl = FALSE;
        }
    }
}

void PreferencesDialog::resetPressure()
{
  minPr = .0f;
  maxPr = 1.f;
  ui.pressureBox_1->setPressure(.0f);
  ui.pressureBox_2->setPressure(.0f);
}
