/***************************************************************************
 *  Copyright (C) 2009 by Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ARRAYGRID_H
#define ARRAYGRID_H

/**
 * Define this to make all patches
 * filled with different colours.
 */
//#define DEBUG_COLORED_PATCHES


#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"

//#ifdef QT_OPENGL_LIB
#   include "GL/glew.h"
#   include "iArrayGrid.h"
#   include "arraygrid_patch.h"


class ArrayGrid : public IArrayGrid,
                  public CRefCounter {
    _GLEW_ENABLED

public:

    virtual int nPatchesX (void) const{
        return m_nPatchesX;
    }
    virtual int nPatchesY (void) const {
        return m_nPatchesY;
    }

    /**
     * Get the patch as IArray interface.
     */
    virtual IArrayGridPatch * patch (int x, int y) {
        return patchptr (x, y);
    }

    /**
     * Get the list of patches that are
     * touched by the given rectanngle.
     * Caller doesn't have to release the returned patches.
     * @returns The number of patches written to dest.
     */
    virtual int touched (const sRect * rect, IArrayGridPatch ** dest);

    /**
     * Get the patch that is touched by
     * the given point.
     * @returns Patch pointer of NULL if out of range.
     */
    virtual IArrayGridPatch * touched (int x, int y);

    /**
     * Convinience function.
     */
    virtual int touched (int x, int y, int w, int h, IArrayGridPatch ** dest);

    /**
     * The width of entire data plane.
     */
    virtual long width (void) const {
        return m_PlaneWidth;
    }

    /**
     * The heigth of entire data plane.
     */
    virtual long height (void) const {
        return m_PlaneHeight;
    }

    /**
     * Size of a single array element.
     */
    virtual long element_size (void) const {
        return m_ElementSz;
    }

    /**
     * Fill all patches with zeros.
     */
    virtual void zero (void);

    /**
     * Fill all patches with given data.
     */
    virtual void fill (BYTE r, BYTE g, BYTE b, BYTE a);

    /**
     * Fill all patches with given data.
     */
    virtual void fill (float r, float g, float b, float a);

    /********* ISPAWNEDOBJECT IMPLANTS *************/

    /**
     * Get the classname for this object.
     */
    virtual const char * myClassName (void) {
        return "Common::ArrayGrid";
    }

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    /**
     * Increase the object's
     * reference counter.
     */
    virtual void addRef (void) {
        CRefCounter::addRef ();
    }

    /**
     * Release the object.
     */
    virtual void release (void) {
        CRefCounter::release ();
    }

    /**
     * Ask the object to emit all
     * notification signals if any.
     */
    virtual void pokeObject (void) {
    }

friend IArrayGrid * g_fac_SPAWN_ARRAYGRID (IOpenGLHelper * helper, LONG width, LONG height, GLenum format, GLenum type);
protected:
    ArrayGrid (IOpenGLHelper * helper, long width, long height, GLenum format, GLenum type);
    virtual ~ArrayGrid (void);

    /**
     * The list of patches. Each patch is
     * a separate IArray object.
     */
    IArray ** m_DataPatches;

    /**
     * Additional data for the patches.
     */
    Patch * m_DataPatches_proxy;


    IArrayGridPatch * patchptr (int x, int y);

private:
    int     m_Id;
    int     m_nPatches;
    int     m_nPatchesX;
    int     m_nPatchesY;
    int     m_PatchSize;
    int     m_PatchDataSize;
    int     m_PlaneWidth;
    int     m_PlaneHeight;
    int     m_ElementSz;

    IOpenGLHelper * m_Helper;

    virtual void suicide (void) {
        delete this;
    }
};

#endif // ARRAYGRID_H
