/***
 *  Copyright (C) 2009 Anton R. <commanderkyle@gmail.com>
 *
 *  This file is a part of QAquarelle project.
 *
 *  QAquarelle is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  QAquarelle is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with QAquarelle.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef BASICDYNAMICS_H
#define BASICDYNAMICS_H

#include "commdefs.h"
#include "commtypes.h"
#include "refcounter.h"

#define ALLOW_INTERFACE_DECLARATION
//#ifdef QT_OPENGL_LIB
#   include "../../../glew/include/GL/glew.h"

#include <QtPlugin>

#include "pipeline_basicdynamics_global.h"
#include "iOpenGLHelper.h"
#include "iLogger.h"
#include "iPipeline.h"


class PIPELINE_BASICDYNAMICSSHARED_EXPORT BasicDynamics  : public QObject,
                                                           public IPipelineFactory,
                                                           public CRefCounter {

    Q_OBJECT
    Q_INTERFACES (IPipelineFactory)

    int m_Id;

    virtual void suicide (void) {
        // Static object?
        //delete this;
    }

protected:

    /**
     * External logger object.
     */
    ILogger * m_Logger;

private:
    /****** IPIPELINEFACTORY IMPLANTS *******/

    /**
     * Get the number of pipelines
     * in this library.
     */
    virtual int pipelinesInLibrary (void) {
        return 1;
    }

    /**
     * Attach the logger object that will
     * be used to log the messages for
     * all allocated pipelines.
     */
    virtual void attachLogger (ILogger * logger) {
        SAFE_RELEASE (m_Logger);

        if (logger) {
            m_Logger = logger;
            m_Logger->addRef ();
        }
    }

public:
    BasicDynamics (void);
    virtual ~BasicDynamics (void);

    /**
     * Create a pipeline.
     * @param index Pipeline index, bounded by the value returned by pipelinesInLibrary ().
     * @param helper OpenGL helper object.
     */
    virtual IPipeline * create (int index, IOpenGLHelper * helper);

    /********** IPLUGIN IMPLANTS ************/

    /**
     * Get the name of this plugin.
     */
    virtual const char * name (void) {
        return "BasicDynamics pipelines factory";
    }

    /**
     * Get the description of this plugin.
     */
    virtual const char * description (void) {
        return "A factory object for a BasicDynamics pipeline pack.";
    }

    /**
     * Get the author of this plugin.
     */
    virtual const char * author (void) {
        return "shadow-of-kyle";
    }

    virtual int customParametersCount (void) const {
        return 0;
    }

    /**
     * Get the name of custom parameter
     * with the given index.
     * /return Pointer to string of NULL if the index is out of range.
     */
    virtual const char * customParameterName (int index) const {
        Q_UNUSED (index);
        return NULL;
    }

    /**
     * Get the type of custom parameter
     * with the given index.
     */
    virtual ePLUGIN_PARAM_TYPE customParameterType (int index) const {
        Q_UNUSED (index);
        return PARAM_UNDEFINED;
    }

    /**
     * Get the type of custom parameter
     * with the given name.
     */
    virtual ePLUGIN_PARAM_TYPE customParameterType (const char * name) const {
        Q_UNUSED (name);
        return PARAM_UNDEFINED;
    }

    /**
     * Get the value of custom parameter with the given name.
     * /return TRUE if the value has been successfully retrieved.
     */
    virtual bool customParameterGet (const char * name, void * buffer) {
        Q_UNUSED (name);
        Q_UNUSED (buffer);
        return FALSE;
    }

    /**
     * Set the value of custom parameter with the given name.
     * /return TRUE if the value has been successfully set.
     */
    virtual bool customParameterSet (const char * name, const void * buffer) {
        Q_UNUSED (name);
        Q_UNUSED (buffer);
        return FALSE;
    }

    /**
     * Deinit all OpenGL-related data
     * structures to be prepared for a
     * context switching.
     */
    virtual void deInit (void) {
    }

    /******* ISPAWNEDOBJECT IMPLANTS *******/

    /**
     * Get the classname for this object.
     */
    virtual const char * myClassName (void) {
        return "Server::BasicDynamics";
    }

    /**
     * Get the Id of current allocated object.
     */
    virtual int objectId (void) {
        return m_Id;
    }

    /**
     * Increase the object's
     * reference counter.
     */
    virtual void addRef (void) {
        CRefCounter::addRef ();
    }

    /**
     * Release the object.
     */
    virtual void release (void) {
        CRefCounter::release ();
    }

    virtual void pokeObject (void) {
    }
};

// #endif // QT_OPENGL_LIB
#endif // BASICDYNAMICS_H
